package com.jpattern.gwt.client.event;

import com.jpattern.gwt.client.ApplicationProxy;
import com.jpattern.gwt.client.IApplicationProvider;
import com.jpattern.gwt.client.command.ACommand;
import com.jpattern.gwt.client.command.ICommandCallBack;
import com.jpattern.gwt.client.logger.ILogger;
import com.jpattern.gwt.client.presenter.IPresenter;
import com.jpattern.shared.result.IResult;

/**
 * 
 * @author Francesco Cina'
 *
 * 14 Apr 2011
 */
public abstract class AEvent<T> implements IEvent<T>, ICommandCallBack {
	
	private final IPresenter presenter;
	private IApplicationProvider provider;
	private IEventCallback<T> eventCallback;
	private final ILogger logger;

	public AEvent(IPresenter presenter) {
		this(presenter, ApplicationProxy.getInstance().getApplicationProvider());
	}
	
	public AEvent(IPresenter presenter, IApplicationProvider provider) {
		this.presenter = presenter;
		this.provider = provider;
		this.logger = getProvider().getLoggerService().getLogger(this.getClass());
	}

	/* (non-Javadoc)
	 * @see com.jpattern.gwt.event.IEvent#launch()
	 */
	@Override
	public final void launch(IEventCallback<T> eventCallback) {
		this.eventCallback = eventCallback;
		logger.debug("launch");
		presenter.onEventStart();
		ACommand command = exec();
		command.visit(getProvider());
		command.exec(this);
	}

	/* (non-Javadoc)
	 * @see com.jpattern.gwt.event.IEvent#callback(com.jpattern.gwt.command.IWebResult)
	 */
	@Override
	public final void callback(IResult commandResult) {
		IEventResult<T> eventResult = afertExec(commandResult);
		logger.debug("callback");
		if ( eventResult.getErrorMessages().size()>0 ) {
			logger.warn("Errors appear: " + eventResult.getErrorMessages());
			presenter.onEventError(eventResult.getErrorMessages());
		}
		eventCallback.callback(eventResult);
	}
	
	protected abstract ACommand exec();
	
	protected abstract IEventResult<T> afertExec(IResult webResult);

	protected IApplicationProvider getProvider() {
		if (provider == null) {
			provider = ApplicationProxy.getInstance().getApplicationProvider();
		}
		return provider;
	}
	
}
