package com.jpattern.gwt.client.history;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import com.jpattern.gwt.client.navigationevent.INavigationEvent;
import com.jpattern.gwt.client.navigationevent.RootNavigationEvent;
import com.jpattern.gwt.client.presenter.IPresenter;
import com.jpattern.gwt.client.presenter.NullPresenter;
import com.jpattern.gwt.client.util.QueryString;

/**
 * 
 * @author Francesco Cina
 *
 * 27/lug/2011
 */
public class HistoryManager implements IHistoryManager {
	
	private static String TOKEN_SEPARATOR = "_";
	
	private int count = 0;
	private final IHistoryManagerObserver historyManagerObserver;
	private final IHistoryEngine historyEngine;
	private RootNavigationEvent rootNavigationEvent = new RootNavigationEvent("");

	public HistoryManager(IHistoryEngine historyEngine, IHistoryManagerObserver historyManagerObserver) {
		this.historyManagerObserver = historyManagerObserver;
		this.historyEngine = historyEngine;
		historyEngine.init(this);
	}

	@Override
	public void onEvent(String eventName) {
		IEventToken eventToken = generateEventToken(eventName);
		String[] tokens = generateTokenList( eventToken.getToken() );
		Map<String, String> queryStringMap = QueryString.toMap(eventToken.getQueryString());
		launchEvents( eventName , tokens , queryStringMap);
	}

	@Override
	public void registerHistory(List<String> presenterHierarchy, Map<String, String> queryStringValues) {
		String token = "";
		if (presenterHierarchy.size()>0 || queryStringValues.size()>0) {
			token = generateTokenString(presenterHierarchy) + count++;
			token = token + QueryString.toQueryString(queryStringValues);
			historyEngine.registerEvent(token);
		}
		historyManagerObserver.onRegisterHistory(token, presenterHierarchy);
	}

	public String generateTokenString(List<String> presenterHierarchy) {
		StringBuffer token = new StringBuffer();
		for (String presenter : presenterHierarchy) {
			token.append( presenter );
			token.append(TOKEN_SEPARATOR);
		}
		return token.toString();
	}
	
	public String[] generateTokenList(String tokenString) {
		String[] result = tokenString.split(TOKEN_SEPARATOR);
		return result;
	}
	
	public void launchEvents(String eventName, String[] tokens , Map<String, String> queryStringMap) {
		List<IPresenter> createdPresenterList = new ArrayList<IPresenter>();
		if (tokens.length<=1){// && tokens[0].equals("")) {
			IPresenter rootPresenter = rootNavigationEvent.launch(new NullPresenter(), queryStringMap);
			createdPresenterList.add(rootPresenter);
			return;
		} else {
			if (tokens.length>1) {
				IPresenter presenter = new NullPresenter();
				INavigationEvent navigationEvent = rootNavigationEvent;
				for (int i=0; i<tokens.length-1; i++) {
					String event = tokens[i+1];
					presenter = navigationEvent.launch(presenter, queryStringMap, event);
					navigationEvent = presenter.getNavigationEventData(event).getNavigationEvent();
					createdPresenterList.add(presenter);
				}
			}
		}
		historyManagerObserver.onEvent(eventName, createdPresenterList);
	}
	
	@Override
	public void updateState() {
		historyEngine.updateState();		
	}

	@Override
	public void setRootNavigationEvent(RootNavigationEvent rootNavigationEvent) {
		this.rootNavigationEvent = rootNavigationEvent;
	}
	
	public IEventToken generateEventToken(String eventName) {
		String queryString = "";
		String token = eventName;
		if (eventName.contains("?")) {
			token = eventName.substring(0, eventName.lastIndexOf("?"));
			queryString = eventName.substring(eventName.lastIndexOf("?"));
		}
		return new EventToken(token, queryString);
	}

}
