package com.jpattern.gwt.client.navigationevent;

import java.util.Map;

import com.jpattern.gwt.client.presenter.IPresenter;

/**
 * 
 * An event to manage the web navigation through managed IPresenter creation 
 * 
 * @author Francesco Cina'
 *
 * 28/07/2011
 */
public interface INavigationEvent {

	/**
	 * Notify a new navigation event to the HistoryManager.
	 * A new presenter instance is created and associated to the parentPresenter.
	 * @param parentPresenter
	 * @param queryStringValues a map with key value pair to be added to the url generated by the navigation event
	 */
	void notifyNavigationEvent(IPresenter parentPresenter, Map<String, String> queryStringValues);
	
	/**
	 * Launch a new navigation event using a specific parentPresenter.
	 * A new presenter instance is created and associated to the parentPresenter.
	 * @param parentPresenter
	 * @param queryStringValues a map with key value pair to be added to the url generated by the navigation event
	 */
	IPresenter launch(IPresenter parentPresenter, Map<String, String> queryStringValues);
	
	/**
	 * Launch a new navigation event using a specific parentPresenter.
	 * A new presenter instance is created and associated to the parentPresenter.
	 * @param parentPresenter
	 * @param queryStringValues a map with key value pair to be added to the url generated by the navigation event
	 * @param childInitialEventName an eventName to set as initial event for the created presenter
	 */
	IPresenter launch(IPresenter parentPresenter, Map<String, String> queryStringValues, String childInitialEventName);

	String getName();
	
}